/**************************************************************************
*  Copyright (c) 2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  30.09.2018  mifi  First Version.
**************************************************************************/
#define __SYSTEM_AM335X_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>

#include "mmu.h"
#include "cache.h"

/*=======================================================================*/
/*  All Extern definitions                                               */
/*=======================================================================*/

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define START_ADDR_DDR        (0x80000000)
#define START_ADDR_DEV        (0x44000000)
#define START_ADDR_OCMC       (0x40300000)
#define NUM_SECTIONS_DDR      (128)
#define NUM_SECTIONS_DEV      (960)
#define NUM_SECTIONS_OCMC     (1)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

uint32_t fnRAMVectors[128];

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static volatile unsigned int pageTable[4*1024] __attribute__ ((section (".mmu")));

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*
** Function to setup MMU. This function Maps three regions (1. DDR
** 2. OCMC and 3. Device memory) and enables MMU.
*/
void MMUConfigAndEnable (void)
{
   /*
    * Default for all regions
    */
   REGION regionDef = {
                       .pgType      = MMU_PGTYPE_SECTION,
                       .startAddr   = 0,
                       .numPages    = 4096,
                       .memAttrib   = 0x12,
                       .secureType  = 0,
                       .accsCtrl    = 0,
                       .masterPtPtr = (unsigned int*)pageTable
                      };

#if 1
    /*
    ** Define DDR memory region of AM335x. DDR can be configured as Normal
    ** memory with R/W access in user/privileged modes. The cache attributes
    ** specified here are,
    ** Inner - Write through, No Write Allocate
    ** Outer - Write Back, Write Allocate
    */
   REGION regionDdr = {
                       .pgType      = MMU_PGTYPE_SECTION,
                       .startAddr   = START_ADDR_DDR,
                       .numPages    = NUM_SECTIONS_DDR,
                       .memAttrib   = MMU_MEMTYPE_NORMAL_SHAREABLE(MMU_CACHE_WT_NOWA, MMU_CACHE_WB_WA),
                       .secureType  = MMU_REGION_NON_SECURE,
                       .accsCtrl    = MMU_AP_PRV_RW_USR_RW,
                       .masterPtPtr = (unsigned int*)pageTable
                      };
#else
   /*
   ** Define DDR memory region of AM335x. DDR can be configured as Normal
   ** memory with R/W access in user/privileged modes. The cache attributes
   ** specified here are,
   ** Inner - Write through, No Write Allocate
   ** Outer - Write Back, Write Allocate
   */
   REGION regionDdr = {
                       MMU_PGTYPE_SECTION, START_ADDR_DDR, NUM_SECTIONS_DDR,
                       MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA,
                                                        MMU_CACHE_WB_WA),
                       MMU_REGION_NON_SECURE, MMU_AP_PRV_RW_USR_RW,
                       (unsigned int*)pageTable

                      };
#endif

   /*
   ** Define OCMC RAM region of AM335x. Same Attributes of DDR region given.
   */
   REGION regionOcmc = {
                        MMU_PGTYPE_SECTION, START_ADDR_OCMC, NUM_SECTIONS_OCMC,
                        MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA,
                                                         MMU_CACHE_WB_WA),
                        MMU_REGION_NON_SECURE, MMU_AP_PRV_RW_USR_RW,
                        (unsigned int*)pageTable
                       };

   /*
   ** Define Device Memory Region. The region between OCMC and DDR is
   ** configured as device memory, with R/W access in user/privileged modes.
   ** Also, the region is marked 'Execute Never'.
   */
   REGION regionDev = {
                       MMU_PGTYPE_SECTION, START_ADDR_DEV, NUM_SECTIONS_DEV,
                       MMU_MEMTYPE_DEVICE_SHAREABLE,
                       MMU_REGION_NON_SECURE,
                       MMU_AP_PRV_RW_USR_RW  | MMU_SECTION_EXEC_NEVER,
                       (unsigned int*)pageTable
                      };

   /* Initialize the page table and MMU */
   MMUInit((unsigned int*)pageTable);

   /* Map the defined regions */
   MMUMemRegionMap(&regionDef);   
   MMUMemRegionMap(&regionDdr);
   MMUMemRegionMap(&regionOcmc);
   MMUMemRegionMap(&regionDev);

   /* Now Safe to enable MMU */
   MMUEnable((unsigned int*)pageTable);
} /* MMUConfigAndEnable */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  SystemInit                                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void SystemInit (void)
{
   MMUConfigAndEnable();
   CacheEnable(CACHE_ALL);
   
} /* SystemInit */   

/*** EOF ***/
